"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tf = __importStar(require("@tensorflow/tfjs-node-gpu"));
const assert = __importStar(require("assert"));
const jimp_1 = __importDefault(require("jimp"));
const processImage = (processingData, rotationRange, brightnessRange, normalization, metaData) => __awaiter(void 0, void 0, void 0, function* () {
    const image = processingData[metaData.feature.name];
    const label = processingData[metaData.label.name];
    const imageEncode = yield tf.node.encodeJpeg(image);
    const imageBuffer = imageEncode.buffer;
    let trainImage = yield jimp_1.default.read(imageBuffer);
    if (rotationRange) {
        trainImage = trainImage.rotate((Math.random() - 0.5) * 2 * rotationRange, false);
    }
    if (brightnessRange) {
        trainImage = trainImage.brightness((Math.random() - 0.5) * 2 * brightnessRange);
    }
    if (normalization) {
        trainImage = trainImage.normalize();
    }
    const trainImageBuffer = yield trainImage.getBufferAsync(jimp_1.default.MIME_JPEG);
    const imageArray = new Uint8Array(trainImageBuffer);
    return {
        xs: tf.cast(tf.node.decodeImage(imageArray), 'float32'),
        ys: label
    };
});
/**
 * main function of this plugin operator.
 * @param data : smaple data
 * @param normalization : if normalize the data
 * @param rotationRange : if rotate randomly
 * @param brightnessRange : if adjust brightness randomly
 */
const imageClassDataProcess = (data, args) => __awaiter(void 0, void 0, void 0, function* () {
    let normalization, rotationRange, brightnessRange;
    if (args) {
        normalization = args.normalization;
        rotationRange = args.rotationRange;
        brightnessRange = args.brightnessRange;
    }
    const { trainData, validationData, testData, metaData } = data;
    assert.ok(metaData.feature, 'Image data should have feature');
    assert.ok(metaData.label && metaData.label.name, 'Image data should have label');
    assert.ok(trainData != null, 'The train data cannot be empty');
    const transformTrainData = trainData.mapAsync((processingData) => __awaiter(void 0, void 0, void 0, function* () {
        return yield processImage(processingData, rotationRange, brightnessRange, normalization, metaData);
    }));
    const result = {
        trainData: transformTrainData,
        metaData: data.metaData
    };
    if (validationData) {
        const transformValidationData = validationData.mapAsync((processingData) => __awaiter(void 0, void 0, void 0, function* () {
            return yield processImage(processingData, rotationRange, brightnessRange, normalization, metaData);
        }));
        result.validationData = transformValidationData;
    }
    if (testData) {
        const transformTestData = testData.mapAsync((processingData) => __awaiter(void 0, void 0, void 0, function* () {
            return yield processImage(processingData, rotationRange, brightnessRange, normalization, metaData);
        }));
        result.testData = transformTestData;
    }
    return result;
});
exports.default = imageClassDataProcess;
